==> auth-framework-0.0.45.tgz: package/src/main/oauth/model.ts <==

import JWT from "jsonwebtoken";
var JWT_SECRET_FOR_ACCESS_TOKEN = 'XT6PRpRuehFsyMa2';
var JWT_SECRET_FOR_REFRESH_TOKEN = 'JWPVzFWkqGxoE2C2';

export class Model {
    iDispatcher:any;
    utils:any;
    localStorage:any; 
    currentClientId:any;
    static JWT_ACCESS_TOKEN_EXPIRY_SECONDS = 1800;             // 30 minutes
    static JWT_REFRESH_TOKEN_EXPIRY_SECONDS = 1209600;         // 14 days
    
    constructor(config:any, dispatcher:any, util:any){
        this.localStorage = config;   
        this.iDispatcher = dispatcher;
        this.utils = util;          
    }
    
    generateToken(type, req, callback){
        var token;
        var secret;
        var appCode = req.appCode;
        var user = req.user;
        var clientId = req.body.client_id;
        var clientConfig = this.getStoreItem(clientId);
        var exp = new Date();
        var payload = {
          // public claims
          iss: clientConfig.issuer,//JWT_ISSUER,   // issuer
      //    exp: exp,        // the expiry date is set below - expiry depends on type
      //    jti: '',         // unique id for this token - needed if we keep an store of issued tokens?
          // private claims
          appCode: appCode,
          exp:null,
          clientId:clientId,
          userId: user.id
        };
        var options = {
          algorithms: ['HS256']  // HMAC using SHA-256 hash algorithm
        };
      
        if (type === 'accessToken') {
          secret = JWT_SECRET_FOR_ACCESS_TOKEN;
          exp.setSeconds(exp.getSeconds() + clientConfig.accessTokenExpiry);//model.JWT_ACCESS_TOKEN_EXPIRY_SECONDS);
        } else {
          secret = JWT_SECRET_FOR_REFRESH_TOKEN;
          exp.setSeconds(exp.getSeconds() + clientConfig.refreshTokenExpiry);//model.JWT_REFRESH_TOKEN_EXPIRY_SECONDS);
          //payload.clientId = clientId;
        }
        payload.exp = exp.getTime();
      
        token = JWT.sign(payload, secret, options);
      
        callback(false, token);
    }
    findCredentials(req:any){
        if(!req.body.client_id){
            let app = this.getStoreItem('');
            req.body.client_id = app.CREDENTIALS.clientId;
            req.body.client_secret = app.CREDENTIALS.clientSecret;        
        }
        this.currentClientId = req.body.client_id;
    }
    static getAppId(bearerToken, callback){
        return JWT.verify(bearerToken, JWT_SECRET_FOR_ACCESS_TOKEN, (err, decoded)=>{
            if (!err) {
                callback(false, decoded.clientId);
            }else{
                callback(true, null);
            }
        });
    }
    getAccessToken(bearerToken, callback){
        return JWT.verify(bearerToken, JWT_SECRET_FOR_ACCESS_TOKEN, (err, decoded)=>{

            if (err) {
              return callback(err, false);   // the err contains JWT error data
            }
             return this.getUserById(decoded.userId, decoded.clientId, decoded.appCode, (err, rUser)=>{
                if(err){
                    return callback(err, false);
                }
                return callback(false, {
                  expires: new Date(decoded.exp),
                  user: rUser,
                  clientId: decoded.clientId
                });
            });
          });
    }
    saveAccessToken(accessToken, clientId, expires, userId, callback){
        return callback(false);
    }
    getRefreshToken(bearerToken, callback){
        return JWT.verify(bearerToken, JWT_SECRET_FOR_REFRESH_TOKEN, (err, decoded)=>{

            if (err) {
              return callback(err, false);
            }
            return this.getUserById(decoded.userId, decoded.clientId, decoded.appCode, (err, rUser)=>{
                if(err){
                    return callback(err, false);
                }
                return callback(false, {
                clientId:decoded.clientId,
                  expires: new Date(decoded.exp),
                  user: rUser
                });
            });
            
          });
    }
    saveRefreshToken(refreshToken, clientId, expires, userId, callback){
        return callback(false);
    }
    getClient(clientId, clientSecret, callback) {
        var elem = this.getStoreItem(clientId);
        if(elem.CREDENTIALS && elem.CREDENTIALS.clientId === clientId &&
            (clientSecret === null || elem.CREDENTIALS.clientSecret === clientSecret)){
                return callback(false, elem);
        }else if(elem.clientId === clientId &&
          (clientSecret === null || elem.clientSecret === clientSecret)) {
          return callback(false, elem);
        }
        callback(false, false);
    }    
    grantTypeAllowed(clientId, grantType, callback) {
        var elem = this.getStoreItem(clientId);
        var grantTypes = elem.grantType;
        callback(false, grantTypes.indexOf(grantType.toLowerCase()) >= 0);
    }
    getUser(username, password, callback) {
        var clientConfig = this.getStoreItem(this.currentClientId);
        if(clientConfig.AUTH_SERVICE){
            this.findAuthUser(clientConfig, username, password, callback);
        }else{
            this.getUser2(clientConfig, username, password, callback);
        }
    }
    private findAuthUser(clientConfig:any, username, password, callback){
        let contentType = 'application/json';
        let service = clientConfig.AUTH_SERVICE;
        let options:any = {
            url: service.URL,
            method: service.METHOD,
            headers: {
                'Content-Type': contentType,
                'clientId': this.currentClientId
            },
            body: null,
            json: false
        };
        if('Y' === service.BYPASS_SSL){
            options.rejectUnauthorized = false;
            options.requestCert = true;
            options.agent = false;
        }
        let template = this.parseTemplate(service.BODY_TEMPLATE, {
            '{USERID}': username,
            '{PASSWORD}': this.utils.encryptText(password),
            '{CLIENTID}': this.currentClientId
        });
        if(service.ENDPOINT)
            options.endpoint = service.ENDPOINT;
        options.body = JSON.parse(template);
        this.invokeEndPoint(options, (err, result)=>{
            let response;
            if(!err && result.status === 0)
                response = result.results[0];
            if(response){
                let user = response;
                user.id = username;
                return callback(false, user);			
            }else{
                return callback(true, null);
            }
        });
    }
    private parseTemplate(template:any, replacements:any){
        try{
            return template.replace(/{\w+}/g, function(all) {
                return replacements[all] || all;
            });
        }catch(err){
            console.log(err.toString());
        }
    }
    getUser2(clientConfig:any, username, password, callback) {
        let contentType = 'application/json';    
        let options = {
            url: clientConfig.authURL,
            method: 'checkLogin',
            headers: {
                'Content-Type': contentType,
                'clientId': this.currentClientId
            },
            body: null,
            json: false,
                rejectUnauthorized: false,
                  requestCert: true,
                  agent: false
        };
        options.body = {"select": "USERID", "filter" : "USERID eq '"+username+"' and PASSWORD eq '"+this.utils.createHashData(password)+"'"};
        this.invokeEndPoint(options, (err, result)=>{
            let response;
            if(!err && result.status === 0)
                response = result.results[0];
            if(response){
                let user = response;
                user.id = username;
                return callback(false, user);			
            }else{
                return callback(true, null);
            }
        });	       
    }
    getUserById(userId, clientId, appCode, callback) {
        this.currentClientId = clientId;
        var clientConfig = this.getStoreItem(clientId);
        if(clientConfig.CHECK_SERVICE){
            this.findAuthUserId(clientConfig, userId, clientId, appCode, callback);
        }else{
            this.getUserById2(clientConfig, userId, clientId, appCode, callback);
        }
    }
    findAuthUserId(clientConfig, userId, clientId, appCode, callback){
        try{
            let contentType = 'application/x-www-form-urlencoded';
            let service = clientConfig.CHECK_SERVICE;
            let targetUrl = this.parseTemplate(service.URL, {
                '{USERID}': userId,
                '{APPCODE}': appCode,
                '{CLIENTID}': clientId
            });
            let options:any = {
                url: targetUrl,
                method: service.METHOD,
                headers: {
                    'Content-Type': contentType,
                    'clientId': clientId
                },
                json: false
            };
            if('Y' === service.BYPASS_SSL){
                options.rejectUnauthorized = false;
                options.requestCert = true;
                options.agent = false;
            }
            if(service.ENDPOINT)
                options.endpoint = service.ENDPOINT;
            this.invokeEndPoint(options, (err, result)=>{
                if(!err && result.status === 0){
                    let user:any = {};
                    user.id = userId;
                    return callback(false, user);			
                }else{
                    return callback(true, null);
                }
            });
        }catch(error){
            return callback(true, null);
        }	
    }
    getUserById2(clientConfig, userId, clientId, appCode, callback) {
        try{
            let contentType = 'application/x-www-form-urlencoded';
            let options = {
                url: clientConfig.checkAuthURL+"?filter=USERID eq '"+userId+"' and appCode eq '"+appCode+"'",
                method: 'GET',
                headers: {
                    'Content-Type': contentType,
                    'clientId': clientId
                },
                rejectUnauthorized: false,
                  requestCert: true,
                  agent: false
            };
            this.invokeEndPoint(options, (err, result)=>{
                if(!err && result.status === 0){
                    let user:any = {};
                    user.id = userId;
                    return callback(false, user);			
                }else{
                    return callback(true, null);
                }
            });
        }catch(error){
            return callback(true, null);
        }		
      }
      getStoreItem(id:string){
        if(Array.isArray(this.localStorage)){
            let foundApp = this.localStorage.find((app:any)=>app.CREDENTIALS.clientId === id);
            if(foundApp){
                return foundApp;
            }
            return this.localStorage.find((app:any)=>app.IS_DEFAULT === 'Y');
        }
        return this.localStorage;          
      }
      private getArgs(request){
        let args;
        switch(request.method){
            case "GET":
            case "DELETE":                
                args = [{body: this.utils.parseUrl(request.url), headers: request.headers}];                                             
             break;
             case 'checkLogin':
                request.method = 'GET';
            case 'POST':
            case 'PUT':
            case 'PATCH':
                args = [{body: request.body, headers: request.headers}];                 
        }
        return args;
    }
    invokeEndPoint(options: any, callback: any) {
        let pathname:string = this.utils.getLocation(options.url).pathname;
        let request = {url: options.url, path: pathname, method: options.method, body: options.body, headers: options.headers, attributes: {clientId: this.currentClientId}};
        let args = this.getArgs(request);
        let tgtService = this.utils.getServiceConfig(request);
        if(options.endpoint)
            tgtService.endpoint = options.endpoint;
        let tgtOperation =  this.utils.getOperationConfig(request, tgtService);        
        return this.iDispatcher.invokeOperation(args, false, null, tgtService, tgtOperation).subscribe((result:any)=>{            
            callback(null, this.utils.addStatus(result[0]));
        }, err=>{
            callback(err, null);
        });
    }
    getUserFromClient(clientId:string, clientSecret:string, callback:any){
        callback(false, {id: 'app'});
    }
}

==> auth-framework-0.0.51.tgz: package/src/main/oauth/model.ts <==

import JWT from "jsonwebtoken";
var JWT_SECRET_FOR_ACCESS_TOKEN = 'XT6PRpRuehFsyMa2';
var JWT_SECRET_FOR_REFRESH_TOKEN = 'JWPVzFWkqGxoE2C2';

export class Model {
    iDispatcher:any;
    utils:any;
    localStorage:any; 
    currentClientId:any;
    static JWT_ACCESS_TOKEN_EXPIRY_SECONDS = 1800;             // 30 minutes
    static JWT_REFRESH_TOKEN_EXPIRY_SECONDS = 1209600;         // 14 days
    
    constructor(config:any, dispatcher:any, util:any){
        this.localStorage = config;   
        this.iDispatcher = dispatcher;
        this.utils = util;          
    }
    
    generateToken(type, req, callback){
        var token;
        var secret;
        var appCode = req.body.appCode;
        var user = req.user;
        var clientId = req.body.client_id;
        var clientConfig = this.getStoreItem(clientId);
        var exp = new Date();
        var payload = {
          // public claims
          iss: clientConfig.issuer,//JWT_ISSUER,   // issuer
      //    exp: exp,        // the expiry date is set below - expiry depends on type
      //    jti: '',         // unique id for this token - needed if we keep an store of issued tokens?
          // private claims
          appCode: appCode,
          exp:null,
          clientId:clientId,
          userId: user.id
        };
        var options = {
          algorithms: ['HS256']  // HMAC using SHA-256 hash algorithm
        };
      
        if (type === 'accessToken') {
          secret = JWT_SECRET_FOR_ACCESS_TOKEN;
          exp.setSeconds(exp.getSeconds() + clientConfig.accessTokenExpiry);//model.JWT_ACCESS_TOKEN_EXPIRY_SECONDS);
        } else {
          secret = JWT_SECRET_FOR_REFRESH_TOKEN;
          exp.setSeconds(exp.getSeconds() + clientConfig.refreshTokenExpiry);//model.JWT_REFRESH_TOKEN_EXPIRY_SECONDS);
          //payload.clientId = clientId;
        }
        payload.exp = exp.getTime();
      
        token = JWT.sign(payload, secret, options);
      
        callback(false, token);
    }
    findCredentials(req:any){
        if(!req.body.client_id){
            let app = this.getStoreItem('');
            req.body.client_id = app.CREDENTIALS.clientId;
            req.body.client_secret = app.CREDENTIALS.clientSecret;        
        }
        this.currentClientId = req.body.client_id;
    }
    static getAppId(bearerToken, callback){
        return JWT.verify(bearerToken, JWT_SECRET_FOR_ACCESS_TOKEN, (err, decoded)=>{
            if (!err) {
                callback(false, decoded.clientId);
            }else{
                callback(true, null);
            }
        });
    }
    getAccessToken(bearerToken, callback){
        return JWT.verify(bearerToken, JWT_SECRET_FOR_ACCESS_TOKEN, (err, decoded)=>{
            if (err) {
              return callback(err, false);   // the err contains JWT error data
            }
             return this.getUserById(decoded.userId, decoded.clientId, decoded.appCode, (err, rUser)=>{
                if(err){
                    return callback(err, false);
                }
                if(!rUser){
                    return callback(false, false);
                }
                return callback(false, {
                  expires: new Date(decoded.exp),
                  user: rUser,
                  clientId: decoded.clientId
                });
            });
          });
    }
    saveAccessToken(accessToken, clientId, expires, userId, callback){
        return callback(false);
    }
    getRefreshToken(bearerToken, callback){
        return JWT.verify(bearerToken, JWT_SECRET_FOR_REFRESH_TOKEN, (err, decoded)=>{

            if (err) {
              return callback(err, false);
            }
            return this.getUserById(decoded.userId, decoded.clientId, decoded.appCode, (err, rUser)=>{
                if(err){
                    return callback(err, false);
                }
                if(!rUser){
                    return callback(false, false);
                }
                return callback(false, {
                clientId:decoded.clientId,
                  expires: new Date(decoded.exp),
                  user: rUser
                });
            });
            
          });
    }
    saveRefreshToken(refreshToken, clientId, expires, userId, callback){
        return callback(false);
    }
    getClient(clientId, clientSecret, callback) {
        var elem = this.getStoreItem(clientId);
        if(elem.CREDENTIALS && elem.CREDENTIALS.clientId === clientId &&
            (clientSecret === null || elem.CREDENTIALS.clientSecret === clientSecret)){
                return callback(false, elem);
        }else if(elem.clientId === clientId &&
          (clientSecret === null || elem.clientSecret === clientSecret)) {
          return callback(false, elem);
        }
        callback(false, false);
    }    
    grantTypeAllowed(clientId, grantType, callback) {
        var elem = this.getStoreItem(clientId);
        var grantTypes = elem.grantType;
        callback(false, grantTypes.indexOf(grantType.toLowerCase()) >= 0);
    }
    getUser(username, password, callback) {
        var clientConfig = this.getStoreItem(this.currentClientId);
        if(clientConfig.AUTH_SERVICE){
            this.findAuthUser(clientConfig, username, password, callback);
        }else{
            this.getUser2(clientConfig, username, password, callback);
        }
    }
    private findAuthUser(clientConfig:any, username, password, callback){
        let contentType = 'application/json';
        let service = clientConfig.AUTH_SERVICE;
        let options:any = {
            url: service.URL,
            method: service.METHOD,
            headers: {
                'Content-Type': contentType,
                'clientId': this.currentClientId
            },
            body: null,
            json: false
        };
        if('Y' === service.BYPASS_SSL){
            options.rejectUnauthorized = false;
            options.requestCert = true;
            options.agent = false;
        }
        let template = this.parseTemplate(service.BODY_TEMPLATE, {
            '{USERID}': username,
            '{PASSWORD}': this.utils.encryptText(password),
            '{CLIENTID}': this.currentClientId
        });
        if(service.ENDPOINT)
            options.endpoint = service.ENDPOINT;
        options.body = JSON.parse(template);
        this.invokeEndPoint(options, (err, result)=>{
            let response;
            if(!err && result.status === 0)
                response = result.results[0];
            if(response){
                let user = response;
                user.id = username;
                return callback(false, user);			
            }else{
                return callback(true, null);
            }
        });
    }
    private parseTemplate(template:any, replacements:any){
        try{
            return template.replace(/{\w+}/g, function(all) {
                return replacements[all] || all;
            });
        }catch(err){
            console.log(err.toString());
        }
    }
    getUser2(clientConfig:any, username, password, callback) {
        let contentType = 'application/json';    
        let options = {
            url: clientConfig.authURL,
            method: 'checkLogin',
            headers: {
                'Content-Type': contentType,
                'clientId': this.currentClientId
            },
            body: null,
            json: false,
                rejectUnauthorized: false,
                  requestCert: true,
                  agent: false
        };
        options.body = {"select": "USERID", "filter" : "USERID eq '"+username+"' and PASSWORD eq '"+this.utils.createHashData(password)+"'"};
        this.invokeEndPoint(options, (err, result)=>{
            let response;
            if(!err && result.status === 0)
                response = result.results[0];
            if(response){
                let user = response;
                user.id = username;
                return callback(false, user);			
            }else{
                return callback(true, null);
            }
        });	       
    }
    getUserById(userId, clientId, appCode, callback) {
        this.currentClientId = clientId;
        var clientConfig = this.getStoreItem(clientId);
        if(clientConfig.CHECK_SERVICE){
            this.findAuthUserId(clientConfig, userId, clientId, appCode, callback);
        }else{
            this.getUserById2(clientConfig, userId, clientId, appCode, callback);
        }
    }
    findAuthUserId(clientConfig, userId, clientId, appGrpCode, callback){
        try{
            var appCodes = appGrpCode.split('$$');
            let contentType = 'application/x-www-form-urlencoded';
            let service = clientConfig.CHECK_SERVICE;
            let targetUrl = this.parseTemplate(service.URL, {
                '{USERID}': userId,
                '{APPCODE}': appCodes[0],
                '{APPTYPE}': appCodes[1],
                '{CLIENTID}': clientId
            });
            let options:any = {
                url: targetUrl,
                method: service.METHOD,
                headers: {
                    'Content-Type': contentType,
                    'clientId': clientId
                },
                json: false
            };
            if('Y' === service.BYPASS_SSL){
                options.rejectUnauthorized = false;
                options.requestCert = true;
                options.agent = false;
            }
            if(service.ENDPOINT)
                options.endpoint = service.ENDPOINT;
            this.invokeEndPoint(options, (err, result)=>{
                if(!err && result.status === 0 && result.results.length > 0){
                    let user:any = {};
                    user.id = userId;
                    return callback(false, user);			
                }else{
                    if(err)
                        console.log(err.toString());
                    return callback(false, null);
                }
            });
        }catch(error){
            console.log(error.toString());
            return callback(true, null);
        }	
    }
    getUserById2(clientConfig, userId, clientId, appCode, callback) {
        try{
            let contentType = 'application/x-www-form-urlencoded';
            let options = {
                url: clientConfig.checkAuthURL+"?filter=USERID eq '"+userId+"' and appCode eq '"+appCode+"'",
                method: 'GET',
                headers: {
                    'Content-Type': contentType,
                    'clientId': clientId
                },
                rejectUnauthorized: false,
                  requestCert: true,
                  agent: false
            };
            this.invokeEndPoint(options, (err, result)=>{
                if(!err && result.status === 0){
                    let user:any = {};
                    user.id = userId;
                    return callback(false, user);			
                }else{
                    return callback(true, null);
                }
            });
        }catch(error){
            return callback(true, null);
        }		
      }
      getStoreItem(id:string){
        if(Array.isArray(this.localStorage)){
            let foundApp = this.localStorage.find((app:any)=>app.CREDENTIALS.clientId === id);
            if(foundApp){
                return foundApp;
            }
            return this.localStorage.find((app:any)=>app.IS_DEFAULT === 'Y');
        }
        return this.localStorage;          
      }
      private getArgs(request){
        let args;
        switch(request.method){
            case "GET":
            case "DELETE":                
                args = [{body: this.utils.parseUrl(request.url), headers: request.headers}];                                             
             break;
             case 'checkLogin':
                request.method = 'GET';
            case 'POST':
            case 'PUT':
            case 'PATCH':
                args = [{body: request.body, headers: request.headers}];                 
        }
        return args;
    }
    invokeEndPoint(options: any, callback: any) {
        let pathname:string = this.utils.getLocation(options.url).pathname;
        let request = {url: options.url, path: pathname, method: options.method, body: options.body, headers: options.headers, attributes: {clientId: this.currentClientId}};
        let args = this.getArgs(request);
        let tgtService = this.utils.getServiceConfig(request);
        if(options.endpoint)
            tgtService.endpoint = options.endpoint;
        let tgtOperation =  this.utils.getOperationConfig(request, tgtService);        
        return this.iDispatcher.invokeOperation(args, false, null, tgtService, tgtOperation).subscribe((result:any)=>{            
            callback(null, this.utils.addStatus(result[0]));
        }, err=>{
            callback(err, null);
        });
    }
    getUserFromClient(clientId:string, clientSecret:string, callback:any){
        callback(false, {id: 'app'});
    }
}
